"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResponsivePopover = exports.Popover = void 0;
const tslib_1 = require("tslib");
/** @jsx jsx */
const react_1 = require("@emotion/react");
const React = tslib_1.__importStar(require("react"));
const Layer_1 = require("./Layer");
const Positions_1 = require("./Positions");
const Tooltip_1 = require("./Tooltip");
const use_focus_trap_1 = require("./Hooks/use-focus-trap");
const react_spring_1 = require("react-spring");
const Sheet_1 = require("./Sheet");
const use_media_1 = require("use-media");
const prop_types_1 = tslib_1.__importDefault(require("prop-types"));
const Providers_1 = require("./Theme/Providers");
const merge_refs_1 = require("./Hooks/merge-refs");
const AnimatedLayer = react_spring_1.animated(Layer_1.Layer);
const Popover = ({ content, children, placement, closeOnMenuItemClick = true, isOpen: defaultShow = false }) => {
    const theme = Providers_1.useTheme();
    const [show, setShow] = React.useState(defaultShow);
    const child = React.Children.only(children);
    const triggerRef = React.useRef(null);
    const popoverRef = React.useRef(null);
    use_focus_trap_1.useFocusElement(popoverRef, show, {
        escapeDeactivates: true,
        clickOutsideDeactivates: true
    });
    const bg = theme.colors.mode === "dark"
        ? theme.colors.background.tint1
        : theme.colors.background.layer;
    function open() {
        setShow(true);
        document.body.addEventListener("click", onBodyClick, false);
        document.body.addEventListener("keydown", onBodyKeyDown, false);
    }
    const unmount = React.useCallback(removeBodyListeners, []);
    React.useEffect(() => unmount, [unmount]);
    function close() {
        setShow(false);
        removeBodyListeners();
    }
    function removeBodyListeners() {
        document.body.removeEventListener("click", onBodyClick, false);
        document.body.removeEventListener("keydown", onBodyKeyDown, false);
    }
    function onBodyClick(e) {
        const trigger = triggerRef.current;
        const popover = popoverRef.current;
        if (!trigger || !popover) {
            return;
        }
        // Don't close if clicking the trigger
        if (trigger === e.target || trigger.contains(e.target)) {
            return;
        }
        // or the popover, sometimes
        if (popover === e.target || popover.contains(e.target)) {
            if (closeOnMenuItemClick) {
                const target = e.target;
                if (target.hasAttribute('[data-trigger-close="true"]') ||
                    target.closest('[data-trigger-close="true"]')) {
                    close();
                    return;
                }
            }
            return;
        }
        close();
    }
    function onBodyKeyDown(e) {
        // close on escape key or enter
        if (e.keyCode === 27) {
            close();
        }
        // close on enter
        if (closeOnMenuItemClick && e.keyCode === 13) {
            close();
        }
    }
    function renderTrigger({ ref }) {
        return React.cloneElement(child, {
            onPress: (e) => {
                onTriggerClicked();
                if (child.props.onPress) {
                    child.props.onPress(e);
                }
            },
            ref: merge_refs_1.mergeRefs(ref, triggerRef),
            role: "button",
            "aria-expanded": show,
            "aria-haspopup": true
        });
    }
    function onTriggerClicked() {
        return show ? close() : open();
    }
    return (react_1.jsx(Positions_1.Positioner, { placement: placement, isOpen: show, target: renderTrigger }, ({ placement, ref, arrowProps, style }, animation) => (react_1.jsx(AnimatedLayer, { role: "dialog", elevation: "md", ref: merge_refs_1.mergeRefs(ref, popoverRef), style: Object.assign(Object.assign({}, style), { opacity: animation.opacity }), "data-placement": placement, css: {
            zIndex: theme.zIndices.popover,
            margin: theme.spaces.sm,
            borderRadius: theme.radii.md,
            background: bg
        } },
        react_1.jsx("div", { "data-placement": placement, css: Tooltip_1.arrowStyles(bg), ref: arrowProps.ref, style: arrowProps.style }),
        content))));
};
exports.Popover = Popover;
exports.Popover.propTypes = {
    isOpen: prop_types_1.default.bool,
    children: prop_types_1.default.node,
    content: prop_types_1.default.node,
    closeOnMenuItemClick: prop_types_1.default.bool,
    placement: prop_types_1.default.oneOf([
        "auto-start",
        "auto",
        "auto-end",
        "top-start",
        "top",
        "top-end",
        "right-start",
        "right",
        "right-end",
        "bottom-end",
        "bottom",
        "bottom-start",
        "left-end",
        "left",
        "left-start"
    ])
};
/**
 * Display popover contents in a bottom sheet if
 * on mobile devices. I generally find that this provides a
 * better use experience on smaller screens.
 */
const ResponsivePopover = (props) => {
    const theme = Providers_1.useTheme();
    // show popover on iPad+
    const showPopover = use_media_1.useMedia({
        minWidth: theme.breakpoints.md
    });
    const [isOpen, setIsOpen] = React.useState(false);
    if (showPopover) {
        return react_1.jsx(exports.Popover, Object.assign({}, props));
    }
    return (react_1.jsx(React.Fragment, null,
        React.cloneElement(React.Children.only(props.children), {
            onPress: () => {
                setIsOpen(true);
            }
        }),
        react_1.jsx(Sheet_1.Sheet, { position: "bottom", isOpen: isOpen, onRequestClose: () => {
                setIsOpen(false);
            } }, props.content)));
};
exports.ResponsivePopover = ResponsivePopover;
exports.ResponsivePopover.propTypes = Object.assign({}, exports.Popover.propTypes);
