"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DarkMode = exports.LightMode = exports.useTheme = exports.ThemeProvider = void 0;
const tslib_1 = require("tslib");
const React = tslib_1.__importStar(require("react"));
const _1 = tslib_1.__importDefault(require("."));
/**
 * API:
 *
 * You can create custom themes using Object.assign:
 *
 *
 * Pass that theme to the theme provider. A theme should always
 * provide a dark and light color mode.
 *
 * <ThemeProvider theme={customTheme}><App /></ThemeProvider>
 *
 *
 * When you want to use a dark mode:
 *
 * function App () {
 *  return <Dark><SomeContent /></Dark>
 * }
 *
 *
 * To consume a theme, use `useTheme` hook
 *
 * const theme = useTheme()
 * // theme.colors.text.default (will be dark or light depending on the mode)
 * // theme.colors.mode === 'dark' or 'light'
 */
const ThemeContext = React.createContext(_1.default);
const ThemeProvider = ({ theme = _1.default, children }) => {
    return (React.createElement(ThemeContext.Provider, { value: theme }, children));
};
exports.ThemeProvider = ThemeProvider;
/**
 * A hook for consuming the theme
 */
function useTheme() {
    return React.useContext(ThemeContext);
}
exports.useTheme = useTheme;
const ColorMode = React.forwardRef((_a, ref) => {
    var { colors, children } = _a, other = tslib_1.__rest(_a, ["colors", "children"]);
    const theme = useTheme();
    // memo is necessary to prevent unnecessary rerenders
    // https://reactjs.org/docs/context.html#caveats
    const adjustedTheme = React.useMemo(() => mergeColors(theme, colors), [
        theme,
        colors
    ]);
    return (React.createElement(ThemeContext.Provider, { value: adjustedTheme }, typeof children === "function"
        ? children(adjustedTheme)
        : React.cloneElement(React.Children.only(children), Object.assign({ ref }, other))));
});
ColorMode.displayName = "ColorMode";
function mergeColors(theme, colors) {
    return Object.assign(Object.assign({}, theme), { shadows: colors.shadows, colors });
}
exports.LightMode = React.forwardRef((_a, ref) => {
    var { children } = _a, other = tslib_1.__rest(_a, ["children"]);
    const theme = useTheme();
    return (React.createElement(ColorMode, Object.assign({ colors: theme.modes.light, ref: ref }, other), children));
});
exports.LightMode.displayName = "LightMode";
/**
 * Provide a dark theme
 */
exports.DarkMode = React.forwardRef((_a, ref) => {
    var { children } = _a, other = tslib_1.__rest(_a, ["children"]);
    const theme = useTheme();
    return (React.createElement(ColorMode, Object.assign({ colors: theme.modes.dark, ref: ref }, other), children));
});
exports.DarkMode.displayName = "DarkMode";
