/** @jsx jsx */
import { jsx } from "@emotion/react";
import * as React from "react";
import { Layer } from "./Layer";
import { Positioner } from "./Positions";
import { arrowStyles } from "./Tooltip";
import { useFocusElement } from "./Hooks/use-focus-trap";
import { animated } from "react-spring";
import { Sheet } from "./Sheet";
import { useMedia } from "use-media";
import PropTypes from "prop-types";
import { useTheme } from "./Theme/Providers";
import { mergeRefs } from "./Hooks/merge-refs";
const AnimatedLayer = animated(Layer);
export const Popover = ({ content, children, placement, closeOnMenuItemClick = true, isOpen: defaultShow = false }) => {
    const theme = useTheme();
    const [show, setShow] = React.useState(defaultShow);
    const child = React.Children.only(children);
    const triggerRef = React.useRef(null);
    const popoverRef = React.useRef(null);
    useFocusElement(popoverRef, show, {
        escapeDeactivates: true,
        clickOutsideDeactivates: true
    });
    const bg = theme.colors.mode === "dark"
        ? theme.colors.background.tint1
        : theme.colors.background.layer;
    function open() {
        setShow(true);
        document.body.addEventListener("click", onBodyClick, false);
        document.body.addEventListener("keydown", onBodyKeyDown, false);
    }
    const unmount = React.useCallback(removeBodyListeners, []);
    React.useEffect(() => unmount, [unmount]);
    function close() {
        setShow(false);
        removeBodyListeners();
    }
    function removeBodyListeners() {
        document.body.removeEventListener("click", onBodyClick, false);
        document.body.removeEventListener("keydown", onBodyKeyDown, false);
    }
    function onBodyClick(e) {
        const trigger = triggerRef.current;
        const popover = popoverRef.current;
        if (!trigger || !popover) {
            return;
        }
        // Don't close if clicking the trigger
        if (trigger === e.target || trigger.contains(e.target)) {
            return;
        }
        // or the popover, sometimes
        if (popover === e.target || popover.contains(e.target)) {
            if (closeOnMenuItemClick) {
                const target = e.target;
                if (target.hasAttribute('[data-trigger-close="true"]') ||
                    target.closest('[data-trigger-close="true"]')) {
                    close();
                    return;
                }
            }
            return;
        }
        close();
    }
    function onBodyKeyDown(e) {
        // close on escape key or enter
        if (e.keyCode === 27) {
            close();
        }
        // close on enter
        if (closeOnMenuItemClick && e.keyCode === 13) {
            close();
        }
    }
    function renderTrigger({ ref }) {
        return React.cloneElement(child, {
            onPress: (e) => {
                onTriggerClicked();
                if (child.props.onPress) {
                    child.props.onPress(e);
                }
            },
            ref: mergeRefs(ref, triggerRef),
            role: "button",
            "aria-expanded": show,
            "aria-haspopup": true
        });
    }
    function onTriggerClicked() {
        return show ? close() : open();
    }
    return (jsx(Positioner, { placement: placement, isOpen: show, target: renderTrigger }, ({ placement, ref, arrowProps, style }, animation) => (jsx(AnimatedLayer, { role: "dialog", elevation: "md", ref: mergeRefs(ref, popoverRef), style: Object.assign(Object.assign({}, style), { opacity: animation.opacity }), "data-placement": placement, css: {
            zIndex: theme.zIndices.popover,
            margin: theme.spaces.sm,
            borderRadius: theme.radii.md,
            background: bg
        } },
        jsx("div", { "data-placement": placement, css: arrowStyles(bg), ref: arrowProps.ref, style: arrowProps.style }),
        content))));
};
Popover.propTypes = {
    isOpen: PropTypes.bool,
    children: PropTypes.node,
    content: PropTypes.node,
    closeOnMenuItemClick: PropTypes.bool,
    placement: PropTypes.oneOf([
        "auto-start",
        "auto",
        "auto-end",
        "top-start",
        "top",
        "top-end",
        "right-start",
        "right",
        "right-end",
        "bottom-end",
        "bottom",
        "bottom-start",
        "left-end",
        "left",
        "left-start"
    ])
};
/**
 * Display popover contents in a bottom sheet if
 * on mobile devices. I generally find that this provides a
 * better use experience on smaller screens.
 */
export const ResponsivePopover = (props) => {
    const theme = useTheme();
    // show popover on iPad+
    const showPopover = useMedia({
        minWidth: theme.breakpoints.md
    });
    const [isOpen, setIsOpen] = React.useState(false);
    if (showPopover) {
        return jsx(Popover, Object.assign({}, props));
    }
    return (jsx(React.Fragment, null,
        React.cloneElement(React.Children.only(props.children), {
            onPress: () => {
                setIsOpen(true);
            }
        }),
        jsx(Sheet, { position: "bottom", isOpen: isOpen, onRequestClose: () => {
                setIsOpen(false);
            } }, props.content)));
};
ResponsivePopover.propTypes = Object.assign({}, Popover.propTypes);
